import HanziWriter, { type HanziWriterOptions } from 'hanzi-writer'
import { type Strokes, createStrokes } from './strokes'
import { colorBg, colorFg, colorHighlight, hanziSize } from '../theme'
import { GetHanziDataKind, getHanziData } from '../hanzi-data'

const globalId = 'strichfolge-animation'

export interface WriterOpts {
  /**
   * Append the component to the end of this container.
   */
  addTo: Element
  /**
   * Text with chinese characters in it. A Hanzi writer will be created for
   * each non-ascii character except `，` (full-width comma). Commas of this
   * kind as well as the standard ASCII comma and spaces will be rendered as
   * simple text without animation.
   */
  text: string
  /**
   * Optional animation settings, e.g. what stroke to animate.
   */
  animate?: AnimateOpts
  /**
   * If set to true, the radical is shown in a different color than the rest.
   */
  highlightRadical?: true
}

const writerOptsDefaults = {
  animate: {}
}

const enum State {
  Loading = 'loading',
  Error = 'error',
  Ready = 'ready'
}

interface Write {
  state: State
}

interface AnimateOpts {
  no?: true
  /**
   * For each character in the text, a single stroke to animate.
   */
  strokes?: number[]
}

interface WriteReady extends Write {
  state: State.Ready
  writers: Writer[]
  /**
   * When selecting a writer to show individual strokes, holds the writer for
   * which this is done. `undefined` if no selection.
   */
  selectedWriter: Writer | undefined
  animatedIdx: number
  animateNextIdx: number
  /**
   * 1 when looping through characters, 0 when only animating the one that is
   * currently selected.
   *
   * Assumed to be zero or positive, otherwise behavior is undefined.
   */
  animateNextIncrement: number
}

interface Writer {
  /** The character used by both the writer and the strokes. */
  character: string
  /** A single tile with an animation of all the strokes for the character */
  hanziWriter: HanziWriter
  /** The strokes as individual SVG tiles. */
  strokes: Strokes
}

/// Adds a Hanzi writer into the element with the ID defined above.
export async function createWriter (userOpts: WriterOpts): Promise<void> {
  const opts = { ...writerOptsDefaults, ...userOpts }
  const container = opts.addTo
  if (!(container instanceof Element)) {
    throw new Error(`Could not find element #${globalId} for the Hanz writer`)
  }
  const writerContainer = document.createElement('div')
  writerContainer.classList.add('writers')
  const strokesContainer = document.createElement('div')
  container.append(writerContainer, strokesContainer)
  const write = await initWrite(writerContainer, strokesContainer, opts)
  animate(write, opts.animate)
  for (const writer of write.writers) {
    writer.hanziWriter.target.node.addEventListener(
      'click', () => { selectWriter(write, writer) }
    )
  }
}

/// Creates the necessary state and DOM elements for the later animations.
async function initWrite (
  writersContainer: HTMLDivElement,
  strokesContainer: HTMLDivElement,
  opts: WriterOpts): Promise<WriteReady> {
  const chars = splitIntoDrawableChars(opts.text)
  const writers: Writer[] = []
  for (let charIdx = 0; charIdx < chars.length; ++charIdx) {
    const char = chars[charIdx]
    if (char === '，' || char === ',') {
      writersContainer.append('，')
    } else if (char.length === 0 || char === ' ') {
      writersContainer.append(' ')
    } else if (char === '/') {
      writersContainer.append(char)
    } else if (/[\u3400-\u9FBF]/.test(char)) {
      const speed = (typeof opts?.animate?.strokes?.length === 'number')
        ? 0.5
        : 1.75
      const hanziWriterOpts: Partial<HanziWriterOptions> = {
        width: hanziSize,
        height: hanziSize,
        padding: 0,
        strokeColor: colorFg,
        outlineColor: colorBg,
        strokeAnimationSpeed: speed,
        renderer: 'canvas'
      }
      if (opts.highlightRadical === true) {
        hanziWriterOpts.radicalColor = colorHighlight
      }
      hanziWriterOpts.charDataLoader = (_char, onComplete, onError) => {
        getHanziData({ char, kind: GetHanziDataKind.ANY }).then(
          data => {
            if (data !== undefined) {
              onComplete(data)
            } else {
              const msg = `No character SVG available for ${char}`
              console.error(msg)
              onError(msg)
            }
          },
          onError
        )
      }
      const hanziWriter = HanziWriter.create(
        writersContainer, char, hanziWriterOpts
      )
      hanziWriter.target.node.classList.add('writer')
      setActive(hanziWriter.target.node, false)
      const strokes = await createStrokes({
        appendTo: strokesContainer,
        character: char
      })
      setActive(strokes.container, false)
      writers.push({
        character: char,
        hanziWriter,
        strokes
      })
    }
  }
  return {
    state: State.Ready,
    writers,
    selectedWriter: undefined,
    animatedIdx: -1,
    animateNextIdx: 0,
    animateNextIncrement: writers.length <= 1 ? 0 : 1
  }
}

/**
 * Selects the specified writer, unless it is already selected. If already
 * selected, de-selects it.
 */
function selectWriter (write: WriteReady, selectedWriter: Writer): void {
  if (write.selectedWriter !== selectedWriter) {
    // new selection: animate that one and show strokes too
    write.selectedWriter = selectedWriter
    write.animateNextIncrement = 0
    write.writers
      .forEach(w => {
        const active = w === selectedWriter
        setActive(w.hanziWriter.target.node, active)
        setActive(w.strokes.container, active)
      })
  } else {
    // was already selected, so de-select all
    write.selectedWriter = undefined
    write.animateNextIncrement = write.writers.length <= 1 ? 0 : 1
    write.writers
      .forEach(w => {
        setActive(w.hanziWriter.target.node, false)
        setActive(w.strokes.container, false)
      })
  }
  // either way continue the animation. if selected, this is looped, if
  // deselected, we animate one more time and then continue advancing
  continueAnimationWith(write, selectedWriter)
}

function setActive (element: Element, active: boolean): void {
  if (active) {
    element.classList.remove('is-inactive')
    element.classList.add('is-active')
  } else {
    element.classList.remove('is-active')
    element.classList.add('is-inactive')
  }
}

function animate (write: WriteReady, opts: AnimateOpts): void {
  if (write.writers.length === 0 || opts.no) {
    return
  }
  let action: () => void
  if ('strokes' in opts && (opts.strokes != null)) {
    const strokes = opts.strokes
    action = () => {
      const writerIdx = next()
      const strokeIdx = strokes[writerIdx]
      if (typeof strokeIdx === 'undefined' || strokeIdx < 0) {
        // fall back to animating the whole character if nothing defined
        const writer = write.writers[writerIdx].hanziWriter
        void writer.animateCharacter({ onComplete: action })
      } else {
        const writer = write.writers[writerIdx]
        // void writer.hanziWriter.setCharacter(writer.character)
        void writer.hanziWriter.animateStroke(strokeIdx, { onComplete: action })
      }
    }
  } else {
    action = () => {
      const writerIdx = next()
      const writer = write.writers[writerIdx]
      void writer.hanziWriter.animateCharacter({ onComplete: action })
    }
  }
  action()

  function next (): number {
    write.animatedIdx = write.animateNextIdx
    write.animateNextIdx += write.animateNextIncrement
    if (write.animateNextIdx < 0) {
      write.animateNextIdx = write.writers.length - 1
    } else if (write.animateNextIdx >= write.writers.length) {
      write.animateNextIdx = 0
    }
    return write.animatedIdx
  }
}

/**
 * Resets the currently running animation and continues with the specified
 * Hanzi writer.
 */
function continueAnimationWith (
  write: WriteReady,
  continueWith: Writer
): void {
  const { writers, animatedIdx } = write
  if (animatedIdx >= 0) {
    const currentWriter = writers[animatedIdx]
    write.animateNextIdx = writers.indexOf(continueWith)
    // reset all current animations, which also triggers the callback
    // to advance the animation to the newly set next index
    void currentWriter.hanziWriter.setCharacter(currentWriter.character)
  }
}

/// Replaces commas with chinese commas and some other gimmicks, then returns
/// an array of strings for which we want to add hanzi writers
function splitIntoDrawableChars (text: string): string[] {
  const charsAll = text.split('')
  const chars: string[] = []
  for (let charIdx = 0; charIdx < charsAll.length; ++charIdx) {
    const char = charsAll[charIdx]
    if (char === '，' || char === ',') {
      chars.push('，')
    } else if (char.length === 0 || char === ' ') {
      chars.push(' ')
    } else {
      chars.push(char)
    }
  }
  return chars
}
